# __init__.py

# import everything needed for a resume
from .address import Address
from .author import Author
from .education import Education
from .experience import Experience
from .project import Project
from .resume import Resume
from .skill import Skill



import os
import subprocess


def xp_fmt(name: str, address: str, spec: str, date: str) -> str:
    """Format experience information for output

    Args:
        name (str): name of experience provider
        address (str): location or link
        spec (str): experience role
        date (str): when experience was gained

    Returns:
        str: html formatted experience info
    """
    return (
        "<div class='xp-h'>"
        + f"<span>{name}</span>"
        + f"<span>{address}</span>"
        + "</div>\n"
        + "<div class='xp-s'>"
        + f"<span>{spec}</span>"
        + f"<span>{date}</span>"
        + "</div>"
    )


def md_to_pdf(filename: str, html_template: str, css_template: str) -> None:
    """Convert markdown to pdf using pandoc and custom templates

    Args:
        filename (str): name of markdown file to convert to pdf
        html_template (str): html template file for pandoc
        css_template (str): css template file for pandoc
    """
    html_template = os.path.abspath(html_template)
    css_template = os.path.abspath(css_template)
    subprocess.run(
        [
            "pandoc",
            f"{filename}.md",
            "-t",
            "html",
            f"--template={html_template}",
            f"--css={css_template}",
            f"--metadata=title:{filename}",
            "--variable",
            "papersize=letter",
            "--variable",
            "margin-top=4",
            "--variable",
            "margin-right=0",
            "--variable",
            "margin-bottom=0",
            "--variable",
            "margin-left=0",
            "--pdf-engine-opt=--enable-local-file-access",
            "-o",
            f"{filename}.pdf",
        ]
    )


def pdf_to_png(filename: str) -> None:
    """Convert pdf to png using poppler utility

    Args:
        filename (str): name of pdf file to convert to png
    """
    # check number of pages in pdf
    pdfinfo = subprocess.check_output(("pdfinfo", f"{filename}.pdf"))
    pages = int(pdfinfo.decode("utf-8").split("Pages: ")[1].split("\n")[0])

    conversion_cmd = ["pdftoppm", "-png", f"{filename}.pdf", filename]

    if pages == 1:
        # output a single image without page number in name
        conversion_cmd.insert(2, "-singlefile")

    subprocess.run(conversion_cmd)


def generate_resume(
    resume, output_dir: str, html_template: str, css_template: str
):
    """Generate resume as markdown, pdf, and png from python script

    Args:
        resume (Resume): resume object generated from python script
        output_dir (str): directory to place .md, .pdf, and .png files in
        html_template (str): html template file for pandoc
        css_template (str): css template file for pandoc

    Returns:
        Resume: resume object is passed through (for printing to console)
    """
    # generate markdown from python resume script
    filename = (
        output_dir.rstrip("/")
        + "/"
        + f"Resume_{resume.author.name.replace(' ', '_')}"
    )
    resume.build(f"{filename}.md")

    # convert markdown to pdf and generate png image
    md_to_pdf(filename, html_template, css_template)
    pdf_to_png(filename)

    return resume



import os
from pathlib import Path


class Resume:
    def __init__(self, author, schema: str):
        self.author = author

        # set up schema
        full_schema = {
            "profile": self.profile,
            "skills": self.skills,
            "experience": self.experience,
            "projects": self.projects,
            "education": self.education,
        }
        self._schema = [
            full_schema[section.strip()] for section in schema.split(",")
        ]

    def build(self, filename: str):
        """Generate resume in markdown format and save to file

        Args:
            filename (str): name of file to save resume to (including path)
        """
        # ensure path exists and file can be created
        path = Path(os.path.abspath(filename))
        path.parent.mkdir(parents=True, exist_ok=True)

        with open(filename, "w") as file:
            file.write(str(self))

    def __str__(self) -> str:
        return "\n\n".join([self.header, *self._schema])

    @property
    def header(self) -> str:
        return repr(self.author)

    @property
    def profile(self) -> str:
        return (
            f"## Career Profile\n\n{self.author.profile}"
            if self.author.profile is not None
            else ""
        )

    @property
    def skills(self) -> str:
        return "## Skills & Interests\n\n" + "  \n".join(
            [str(skill) for skill in self.author.skills]
        )

    @property
    def experience(self) -> str:
        return "## Work Experience\n\n" + "\n\n".join(
            [str(experience) for experience in self.author.experiences]
        )

    @property
    def projects(self) -> str:
        return "## Project Experience\n\n" + "\n\n".join(
            [str(project) for project in self.author.projects]
        )

    @property
    def education(self) -> str:
        return f"## Education\n\n{self.author.education}"



#!/usr/bin/env python3

import argparse
import os
import sys
from importlib.machinery import SourceFileLoader
from importlib.util import module_from_spec, spec_from_loader

from ..components.resume import Resume
from ..utils import generate_resume


def main(args: argparse.Namespace):
    # set paths to resume file
    full_path = os.path.abspath(args.resume)
    filename = os.path.basename(full_path)
    sys.path.append(os.path.dirname(full_path))

    # import resume file as module
    loader = SourceFileLoader(filename, full_path)
    spec = spec_from_loader(filename, loader)
    resume_info = module_from_spec(spec)
    loader.exec_module(resume_info)

    # use author info from resume module to generate resume
    resume_obj = Resume(resume_info.author, args.schema)
    generate_resume(
        resume_obj, args.output, args.html_template, args.css_template
    )


def parse_args() -> argparse.Namespace:
    # resolve path to default template
    template_dir = os.path.realpath(
        os.path.join(
            os.path.dirname(os.path.realpath(__file__)), "../template"
        )
    )

    parser = argparse.ArgumentParser(
        prog="resumake.builder",
        description="Generate a resume from a given Python file containing data representation objects",
        allow_abbrev=False,
    )
    parser.add_argument(
        "resume",
        metavar="RESUME",
        type=str,
        help="path to the Python file containing the resume data",
    )
    parser.add_argument(
        "-s",
        "--schema",
        dest="schema",
        type=str,
        help="order of sections to display in the resume",
        default="skills, experience, projects, education",
    )
    parser.add_argument(
        "-o",
        "--output",
        dest="output",
        type=str,
        help="path to the output directory",
        default="./",
    )
    parser.add_argument(
        "-t",
        "--template",
        metavar="HTML",
        dest="html_template",
        type=str,
        help="path to the HTML template file",
        default=f"{template_dir}/resume.html",
    )
    parser.add_argument(
        "-c",
        "--css",
        metavar="CSS",
        dest="css_template",
        type=str,
        help="path to the CSS template file",
        default=f"{template_dir}/resume.css",
    )
    return parser.parse_args()


if __name__ == "__main__":
    main(args=parse_args())
